/*
 / _____)             _              | |
( (____  _____ ____ _| |_ _____  ____| |__
 \____ \| ___ |    (_   _) ___ |/ ___)  _ \
 _____) ) ____| | | || |_| ____( (___| | | |
(______/|_____)_|_|_| \__)_____)\____)_| |_|
    (C)2015 Semtech

Description: End device commissioning parameters

License: Revised BSD License, see LICENSE.TXT file include in the project

Maintainer: Miguel Luis and Gregory Cristian
*/
#ifndef __LORA_COMMISSIONING_H__
#define __LORA_COMMISSIONING_H__

/*!
 * When set to 1 the application uses the Over-the-Air activation procedure
 * When set to 0 the application uses the Personalization activation procedure
 *
 */
#define OVER_THE_AIR_ACTIVATION                     0

/*!
 * Indicates if the end-device is to be connected to a private or public network
 */
#define LORAWAN_PUBLIC_NETWORK                      true

/*!
 * IEEE Organizationally Unique Identifier ( OUI ) (big endian)
 */
#define IEEE_OUI                                    0x00, 0x00, 0x00//0xAA,0xBB,0xCC

/*!
 * Mote device IEEE EUI (big endian)
 *
 * \remark In this application the value is automatically generated by calling
 *         BoardGetUniqueId function
 */
#define LORAWAN_DEVICE_EUI                          { IEEE_OUI, 0x07, 0x02, 0x03, 0x04, 0x05 }//{ IEEE_OUI, 0x07, 0x02, 0x03, 0x04, 0x05 }

/*!
 * Application IEEE EUI (big endian)
 */
#define LORAWAN_APPLICATION_EUI                     { 0x28, 0x88, 0xF3, 0x70, 0xEA, 0xBB, 0xA0, 0xB0 }
//network name : lora_gateway

/*!
 * AES encryption/decryption cipher application key
 */
#define LORAWAN_APPLICATION_KEY                     { 0xB0, 0x8C, 0x1B, 0x98, 0xFD, 0x48, 0xC0, 0xEC, 0x29, 0xC7, 0x49, 0xB2, 0x02, 0x7D, 0x22, 0xBD }

/*!
 * Current network ID
 */
#define LORAWAN_NETWORK_ID                          ( uint32_t )0

/*!
 * Device address on the network (big endian)
 *
 * \remark In this application the value is automatically generated using
 *         a pseudo random generator seeded with a value derived from
 *         BoardUniqueId value if LORAWAN_DEVICE_ADDRESS is set to 0
 */
#define LORAWAN_DEVICE_ADDRESS                      ( uint32_t )0x00000001;

/*!
 * AES encryption/decryption cipher network session key
 */
#define LORAWAN_NWKSKEY 	{ 0xED, 0xB1, 0xED, 0x68, 0x5F, 0x79, 0x6E, 0x1A, 0xA1, 0xA8, 0xB9, 0xBC, 0xFC, 0x66, 0x71, 0x4D}

/*!
 * AES encryption/decryption cipher application session key
 */
#define LORAWAN_APPSKEY 	{ 0xDB, 0x21, 0xAC, 0x80, 0x7B, 0x14, 0x7E, 0x99, 0x1E, 0xD0, 0xB4, 0x4E, 0x42, 0x6D, 0xCA, 0xDA}


#endif // __LORA_COMMISSIONING_H__
